package minefield

import "rand"

type Minefield struct {
	Rows, Cols, Mines int32
	// The possible values of each field byte:
	//   ? - unknown field
	//   0..8 - known field
	//   x - field with a mine
	Fields [][]byte
}

func Create(rows, cols, mines int32) (field *Minefield) {
	field = new(Minefield)
	field.Rows, field.Cols, field.Mines = rows, cols, mines
	initialize(field)
	generate_mines(field)
	put_nums(field)
	return
}

func ToByteSlice(field *Minefield, bytes *[]byte) {
	*bytes = make([]byte, field.Rows*field.Cols)
	i := 0
	for r := range field.Fields {
		for c := range field.Fields[r] {
			(*bytes)[i] = field.Fields[r][c]
			i += 1
		}
	}
}

// Assumes that field has the correct Rows, Cols and Mines values
func FromByteSlice(bytes []byte, field *Minefield) {
	i := 0
	for r := range field.Fields {
		for c := range field.Fields[r] {
			field.Fields[r][c] = bytes[i]
			i += 1
		}
	}
}

func initialize(field *Minefield) {
	field.Fields = make([][]byte, field.Rows)
	for i := range field.Fields {
		field.Fields[i] = make([]byte, field.Cols)
	}
}

func generate_mines(field *Minefield) {
	for m := 0; int32(m) < field.Mines; m++ {
		r, c := rand.Int31n(field.Rows), rand.Int31n(field.Cols)
		if field.Fields[r][c] != 0 {
			m -= 1
		} else {
			field.Fields[r][c] = 'x'
		}
	}
}

func is_mine_at(r, c int, field *Minefield) bool {
	if (r < 0 || int32(r) >= field.Rows) {
		return false
	}
	if (c < 0 || int32(c) >= field.Cols) {
		return false
	}
	if (field.Fields[r][c] == 'x') {
		return true
	}
	return false
}

func put_nums(field *Minefield) {
	for r := 0; int32(r) < field.Rows; r++ {
		for c := 0; int32(c) < field.Cols; c++ {
			if field.Fields[r][c] == 'x' {
				continue
			}
			cnt := 0;
			for i := -1; i <= 1; i++ {
				for j := -1; j <= 1; j++ {
					if is_mine_at(r+i, c+j, field) {
						cnt += 1;
					}
				}
			}
			field.Fields[r][c] = byte(cnt)
		}
	}
}
