package network_structs

import "fmt"
import "net"
import "os"

const Port = 12345

// This is the first structure sent to each player.
type GameDescription struct {
	Rows, Cols, Mines int32 "the dimensions of the game field"
	Player_id int32 "id of the player to whom this struct is sent"
}

func SendGameDescription(desc GameDescription, conn net.Conn) {
	send_int32(desc.Rows, conn)
	send_int32(desc.Cols, conn)
	send_int32(desc.Mines, conn)
	send_int32(desc.Player_id, conn)
}

func RecvGameDescription(conn net.Conn) (desc GameDescription) {
	desc.Rows = recv_int32(conn)
	desc.Cols = recv_int32(conn)
	desc.Mines = recv_int32(conn)
	desc.Player_id = recv_int32(conn)
	return
}

// Each other structure sent to the players if the following.
type Field struct {
	Next_player int32 "id of the player who can move next"
	Scores[2] int32 "number of mines found by the players"
	// The possible values of each field byte:
	//   ? - unknown field
	//   0..8 - known field
	//   x - field with a mine
	Fields []byte
}

func SendField(field *Field, conn net.Conn) {
	send_int32(field.Next_player, conn)
	send_int32(field.Scores[0], conn)
	send_int32(field.Scores[1], conn)
	send_n(len(field.Fields), field.Fields, conn)
}

// The field.Fields must be correctly sized in order to work!
func RecvField(field *Field, conn net.Conn) {
	field.Next_player = recv_int32(conn)
	field.Scores[0] = recv_int32(conn)
	field.Scores[1] = recv_int32(conn)
	recv_n(len(field.Fields), field.Fields, conn)
}

// This is sent to the server by the players.
type Move struct {
	Row, Col int32 "the coordinates of the move"
}

func SendMove(move Move, conn net.Conn) {
	send_int32(move.Row, conn)
	send_int32(move.Col, conn)
}

func RecvMove(conn net.Conn) (move Move) {
	move.Row = recv_int32(conn)
	move.Col = recv_int32(conn)
	return
}

func send_n(n int, b []byte, conn net.Conn) {
	sent := 0
	for sent < n {
		w, error := conn.Write(b[sent:n-sent])
		check_error(error);
		sent += w
	}
}

func recv_n(n int, b []byte, conn net.Conn) {
	read := 0
	for read < n {
		r, error := conn.Read(b[read:n-read])
		check_error(error);
		read += r
	}
}

func send_int32(n int32, conn net.Conn) {
	b := tobyte4(n)
	send_n(4, b[:], conn)
}

func recv_int32(conn net.Conn) int32 {
	var b[4] byte
	recv_n(4, b[:], conn)
	return frombyte4(b)
}

func tobyte4(n int32) [4]byte {
	var b[4] byte
	b[0] = uint8((n >>  0) & 0xFF)
	b[1] = uint8((n >>  8) & 0xFF)
	b[2] = uint8((n >> 16) & 0xFF)
	b[3] = uint8((n >> 24) & 0xFF)
	return b
}

func frombyte4(b[4] byte) int32 {
	var n int32
	n += int32(b[0]) <<  0;
	n += int32(b[1]) <<  8;
	n += int32(b[2]) << 16;
	n += int32(b[3]) << 24;
	return n
}

func check_error(err os.Error) {
	if err != nil {
		fmt.Printf("Error: %s\n", err.String())
		panic(err.String())
	}
}
